#ifndef HAVE_XINE_H
#define HAVE_XINE_H

#include "inttypes.h"
#include "input/input_plugin.h"

/*
 * player status constants:
 */

#define XINE_STOP      0 
#define XINE_FILLBUF   1 
#define XINE_PLAY      2 
#define XINE_PAUSE     3 
#define XINE_QUIT      4

/* nStatus : current xine status */
typedef void (*gui_status_callback_func_t)(int nStatus);

/*
 * init xine - call once at startup
 *
 * no_xv               : don't use Xv for video output
 * gui_status_callback : function to call on stream end (for example)
 * audio_driver_id     : suggested audio driver
 * ao_mode             : mode in which to use audio driver
 */

void xine_init (int no_xv, gui_status_callback_func_t gui_status_callback, 
		int audio_driver_id, int ao_mode);

/*
 * open an mpeg stream and play it
 *
 * name : mrl to open
 * pos  : start position 0..65535
 *
 */

void xine_play (char *MRL, int pos);


/*
 * toggle pause mode
 */

void xine_pause ();


/*
 * stop playing
 */

void xine_stop ();


/*
 * Eject media.
 */
int xine_eject(char *);


/*
 * exit xine
 */

void xine_exit ();


/*
 * get player status
 */

int xine_get_status ();


/*
 * return the current audio channel
 */

int xine_get_audio_channel (void);


/*
 * set desired audio channel
 */

void xine_select_audio_channel (int nChannel);


/*
 * return the current SPU channel
 */

int xine_get_spu_channel (void);


/*
 * set desired SPU channel
 */

void xine_select_spu_channel (int nChannel);


/*
 *
 */

input_plugin_t* xine_get_input_plugin_list (int *nInputPlugins); 


/*
 * Get current position in stream
 */

int xine_get_current_position (void);


/*
 * Return current status (XINE_PLAY/XINE_STOP...)
 */

int xine_get_status(void);


/*
 * TODO / Ideas:
 *
 * set volume / bright / contrast ...
 *
 */

/*
 * play(and more) from command line
 */
uint32_t  xine_user_pref;
#define PLAY_ON_START      0x00000001
#define PLAYED_ON_START    0x00000002
#define QUIT_ON_STOP       0x00000004
#define FULL_ON_START      0x00000008
#define HIDEGUI_ON_START   0x00000010
#define PLAY_FROM_DVD      0x00000020
#define PLAY_FROM_VCD      0x00000040
/*
 * our own xlock function - some platforms
 * dont have multithreading support in xlib
 */

/* 
#define XLOCK() { printf ("XLOCK\n"); pthread_mutex_lock (&gXLock); }
#define XUNLOCK() { printf ("XUNLOCK\n");pthread_mutex_unlock (&gXLock); }
*/
#define XLOCK() { XLockDisplay (gDisplay); }
#define XUNLOCK() { XUnlockDisplay (gDisplay); }

#endif
